@@ -11,7 +11,7 @@
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: memory.h,v 1.50.2.2 2004/11/22 11:16:03 doligez Exp $ */
+/* $Id: memory.h,v 1.54 2005/10/14 16:40:48 xleroy Exp $ */
 
 /* Allocation macros and functions */
 
@@ -45,8 +45,8 @@ color_t caml_allocation_color (void *hp);
 
 struct caml__roots_block {
   struct caml__roots_block *next;
-  long ntables;
-  long nitems;
+  intnat ntables;
+  intnat nitems;
   value *tables [5];
 };
 
@@ -109,7 +109,7 @@ CAMLextern struct caml__roots_block *caml_local_roots;  /* defined in roots.c */
   CAMLxparamN (x, (size))
 
 
-#if defined (__GNUC__)
+#if defined (__GNUC__) && (__GNUC__ > 2 || (__GNUC__ == 2 && __GNUC_MINOR__ > 7))
   #define CAMLunused __attribute__ ((unused))
 #else
   #define CAMLunused
